@@ -49,18 +49,19 @@ class Tests(unittest.TestCase):
                 title_font="Pump Triline",
                 title_color="#e69300",
                 plot_label="Surface",
+                background_color_plot="#fff6e6",
+                edge_color="lightgrey", edges_to_rgba=True,
                 azim=-160,
                 elev=43)
-        # line(fig=fig, ax=ax2, color="#ffbf87",
-        #      plot_label="Line", grid=True)
         fill_area(fig=fig, ax=ax2,
-                  plot_label="Fill", aspect=12, grid=False)
+                  plot_label="Fill", aspect=1, grid=False)
         quiver(fig=fig, ax=ax3,
                plot_label="Quiver", grid=False)
         heatmap(fig=fig, ax=ax4,
                 title="No label",
                 plot_label="Heatmap", grid=False)
         streamline(fig=fig, ax=ax5,
+                   line_density=1,
                    title="No label",
                    plot_label="Streamline", grid=False)
 
